import os


def find_file_across_path(dirname, filename):
    filepath = os.path.join(dirname, filename)
    if os.path.exists(filepath):
        return filepath

    new_dirname = os.path.abspath(os.path.join(dirname, '..'))

    if dirname == new_dirname:
        raise RuntimeError('File not found!', filename)

    return find_file_across_path(new_dirname, filename)



import os
try:
    from ConfigParser import ConfigParser, NoSectionError, NoOptionError
except ImportError:
    from configparser import ConfigParser, NoSectionError, NoOptionError

from .utilities import find_file_across_path

FILE_TO_CHECK = 'setup.cfg'
CONFIG_SECTION = 'pyhistory'


def get_defaults_from_config_file_if_exists(file_to_check=FILE_TO_CHECK):
    keys = [
        'history_dir',
        'history_file',
        'at_line',
    ]

    try:
        config_file = find_file_across_path(os.getcwd(), file_to_check)
    except RuntimeError:
        return {key: None for key in keys}

    config = _get_config_from_file(config_file)
    return {key: config.get_or_default(key) for key in keys}

def _get_config_from_file(config_file):
    parser = ConfigParser()
    parser.read(config_file)
    return _ConfigGetter(parser, CONFIG_SECTION)


class _ConfigGetter(object):

    def __init__(self, parser, section):
        self.parser = parser
        self.section = section

    def get_or_default(self, key, default=None):
        try:
            return self.parser.get(self.section, key)
        except (NoSectionError, NoOptionError):
            return default



import os
import time
import unittest
import shutil

from invoke import run as original_run

from pyhistory.pyhistory import select_dir_with_file
from pyhistory.file_config import get_defaults_from_config_file_if_exists

FIXTURES_DIR_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    'fixtures'
)
TEST_DIR = 'test_dir'
TEST_DIR_PATH = os.path.join(os.getcwd(), TEST_DIR)


def run(command):
    return original_run(command, hide=True)


class TestPyhistory(unittest.TestCase):

    def setUp(self):
        try:
            shutil.rmtree(TEST_DIR)
        except OSError:
            pass
        os.mkdir(TEST_DIR)

        self.original_working_dir = os.getcwd()
        os.chdir(TEST_DIR)

    def tearDown(self):
        os.chdir(self.original_working_dir)
        shutil.rmtree(TEST_DIR)

    def test_list_empty(self):
        open('HISTORY.rst', 'w').close()
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(['', '']))

    def test_add_list_and_clear(self):
        open('HISTORY.rst', 'w').close()

        run('pyhi add some_message')
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* some_message', '']))

        _sleep()

        run('pyhi add "next message"')
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* some_message', '* next message', '']))

        run('pyhi clear')
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(['', '']))

    def test_update(self):
        self._test_update('update')

    def test_squash(self):
        self._test_update('squash')

    def _test_update(self, command):
        _load_fixture('history1.rst', 'HISTORY.rst')
        run('pyhi add some_message')
        _sleep()
        run('pyhi add "next message"')
        run('pyhi {} 1.0.6 --date today'.format(command))
        self.assertEqual(
            _get_fixture_content('history1_after.rst'),
            _get_test_file_content('HISTORY.rst')
        )

    def test_update_at_line(self):
        self._test_update_at_line('update')

    def test_squash_at_line(self):
        self._test_update_at_line('squash')

    def _test_update_at_line(self, command):
        _load_fixture('history1.rst', 'HISTORY.rst')
        run('pyhi add some_message')
        _sleep()
        run('pyhi add "next message"')
        run('pyhi {} 1.0.6 --date today --at-line 1'.format(command))
        self.assertEqual(
            _get_fixture_content('history1_at_line_after.rst'),
            _get_test_file_content('HISTORY.rst')
        )

        _load_fixture('history1.rst', 'HISTORY.rst')
        run('pyhi add some_message')
        _sleep()
        run('pyhi add "next message"')
        run('pyhi {} 1.0.6 --date today --at-line 0'.format(command))
        self.assertEqual(
            _get_fixture_content('history1_at_line_after.rst'),
            _get_test_file_content('HISTORY.rst')
        )

        _load_fixture('history1.rst', 'HISTORY.rst')
        run('pyhi add some_message')
        _sleep()
        run('pyhi add "next message"')
        run('pyhi {} 1.0.6 --date today --at-line 7'.format(command))
        self.assertEqual(
            _get_fixture_content('history1_at_line_after2.rst'),
            _get_test_file_content('HISTORY.rst')
        )

    def test_pyhistory_when_not_in_history_file_directory(self):
        _load_fixture('history1.rst', 'HISTORY.rst')

        original_working_dir = os.getcwd()
        os.makedirs('one/two')
        os.chdir('one/two')

        run('pyhi add some_message')
        _sleep()
        run('pyhi add "next message"')
        _sleep()

        self.assertEqual(0, len(os.listdir(os.getcwd())))

        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* some_message', '* next message', '']))

        os.chdir(original_working_dir)
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* some_message', '* next message', '']))

        os.chdir('one/two')
        run('pyhi update 1.0.6 --date today')
        os.chdir(original_working_dir)

        self.assertEqual(
            _get_fixture_content('history1_after.rst'),
            _get_test_file_content('HISTORY.rst')
        )

    def test_select_dir_with_file(self):
        os.makedirs('one/two')
        os.chdir('one')
        proper_dir = os.getcwd()
        open('some_file.rst', 'w').close()
        os.chdir('two')

        selected_dir = select_dir_with_file(
            os.path.join(os.getcwd(), 'some_file.rst'))

        self.assertEqual(proper_dir, selected_dir)

    def test_select_dir_with_fails_when_not_found(self):
        self.assertRaises(
            RuntimeError, select_dir_with_file, '/!hope_this_not_exists')

    def test_delete(self):
        _load_fixture('history1.rst', 'HISTORY.rst')

        run('pyhi add some_message')
        _sleep()
        run('pyhi add "next message"')
        _sleep()

        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* some_message', '* next message', '']))

        result = run('pyhi delete')
        self.assertEqual(result.stdout, _join_lines([
            '', '1. some_message', '2. next message', '',
            '(Delete by choosing entries numbers.)',
        ]))

        run('pyhi delete 1')
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* next message', '']))

        run('pyhi add test')
        _sleep()
        run('pyhi add test2')

        result = run('pyhi delete')
        self.assertEqual(result.stdout, _join_lines([
            '', '1. next message', '2. test', '3. test2', '',
            '(Delete by choosing entries numbers.)',
        ]))

        run('pyhi delete 10')
        result = run('pyhi delete')
        self.assertEqual(result.stdout, _join_lines([
            '', '1. next message', '2. test', '3. test2', '',
            '(Delete by choosing entries numbers.)',
        ]))

        run('pyhi delete 2 3 5 101')
        result = run('pyhi list')
        self.assertEqual(result.stdout, _join_lines(
            ['', '* next message', '']))

    def test_delete_in_non_root(self):
        os.makedirs('one/two')
        os.chdir('one')
        self.test_delete()

    def test_get_config_from_file(self):
        _load_fixture('setup.cfg', 'empty.txt')
        pattern = {
            'history_dir': None,
            'history_file': None,
            'at_line': None,
        }
        self.assertEqual(pattern, get_defaults_from_config_file_if_exists())

    def test_load_config_from_setup_cfg(self):
        _load_fixture('setup.cfg', 'setup.cfg')
        pattern = {
            'history_dir': None,
            'history_file': 'HISTORY.rst',
            'at_line': '42',
        }
        self.assertEqual(pattern, get_defaults_from_config_file_if_exists())

    def test_load_config_when_file_doesnt_exist(self):
        pattern = {
            'history_dir': None,
            'history_file': None,
            'at_line': None,
        }
        self.assertEqual(
            pattern, get_defaults_from_config_file_if_exists('!wrong'))

def _get_test_file_content(name):
    with open(os.path.join(TEST_DIR_PATH, name)) as test_file:
        return test_file.read()


def _get_fixture_content(name):
    with open(os.path.join(FIXTURES_DIR_PATH, name)) as fixture:
        return fixture.read()


def _load_fixture(fixture_name, destination):
    with open(os.path.join(FIXTURES_DIR_PATH, fixture_name)) as fixture:
        with open(os.path.join(TEST_DIR_PATH, destination), 'w') as dest:
            dest.write(fixture.read())


def _join_lines(output):
    return '\n'.join(output) + '\n'


def _sleep():
    time.sleep(0.001)



import argparse

from . import __version__ as ver
from . import pyhistory, file_config

# Default values.
default_values = file_config.get_defaults_from_config_file_if_exists()
to_override_if_none = {
    'history_dir': 'history',
    'history_file': 'HISTORY.rst',
}

for key, value in to_override_if_none.items():
    default_values[key] = default_values[key] or value

# General parser.
parser = argparse.ArgumentParser(
    description="Manage Python project history file.")
parser.add_argument(
    '--version', action='version', version="Pyhistory ver {}".format(ver))
parser.add_argument('--history-dir', default=default_values['history_dir'])
parser.add_argument('--history-file', default=default_values['history_file'])

subparsers = parser.add_subparsers(help="sub-command help")

# Add.
parser_add = subparsers.add_parser('add', help="add new message")
parser_add.add_argument('message')
parser_add.set_defaults(func=pyhistory.add)

# List.
parser_list = subparsers.add_parser('list', help="list actual history")
parser_list.set_defaults(func=pyhistory.list_history)

# Update and squash.
update_parsers = [
    subparsers.add_parser('update', help='update history file'),
    subparsers.add_parser('squash', help='alias to "update"'),
]

for uparser in update_parsers:
    uparser.add_argument('version')
    uparser.add_argument('--date', help="date of release (by default today)")
    uparser.add_argument(
        '--at-line',
        help="at which line put history in history file",
        default=default_values['at_line']
    )
    uparser.set_defaults(func=pyhistory.update)

# Clear.
parser_clear = subparsers.add_parser(
    'clear', help="remove all entries from history directory")
parser_clear.set_defaults(func=pyhistory.clear)

# Delete.
parser_delete = subparsers.add_parser(
    'delete', help="remove specified entries from history directory")
parser_delete.add_argument(
    'entry', help='Entries to delete', nargs='*')
parser_delete.set_defaults(func=pyhistory.delete)


def main():
    args = parser.parse_args()
    args.func(args)



from __future__ import print_function

import os.path
import shutil
import time
from itertools import count, chain
from datetime import date
from hashlib import md5

from .utilities import find_file_across_path


def add(args):

    history_dir = _select_history_dir(args)

    _check_history_dir(history_dir)
    hashed = '{}-{}'.format(
        int(time.time() * 1000),
        md5(args.message.encode('utf-8')).hexdigest()[:7])

    filepath = os.path.join(history_dir, hashed)
    if os.path.exists(filepath):
        raise RuntimeError("Collision, you lucky bastard!")

    with open(filepath, 'w') as entry_file:
        entry_file.write(args.message + '\n')


def list_history(args):
    history_dir = _select_history_dir(args)
    lines = ['* {}'.format(line) for line in _list_history_lines(history_dir)]
    print('\n' + ''.join(lines))


def update(args):
    history_dir = _select_history_dir(args)
    history_file = os.path.join(history_dir, '..', args.history_file)

    with open(history_file) as hfile:
        lines = hfile.readlines()

    break_line = _calculate_break_line(args, lines)
    result = lines[:break_line]

    release_date = args.date or date.today().strftime('%Y-%m-%d')
    header = '{} ({})'.format(args.version, release_date)
    result.append(header + '\n')
    result.append('+' * len(header) + '\n\n')

    new_lines = [
        '* {}'.format(line) for line in _list_history_lines(history_dir)]
    result += new_lines
    result.append('\n')

    result += lines[break_line:]

    result = ''.join(result)

    with open(history_file, 'w') as hfile:
        hfile.write(result)

    _delete_history_dir(history_dir)


def _calculate_break_line(args, lines):
    if args.at_line:
        return max(0, int(args.at_line) - 1)

    start = 0
    for line in lines:
        if not line.startswith('..') and line != '\n':
            break
        start += 1

    return start + 3


def clear(args):
    history_dir = _select_history_dir(args)
    _delete_history_dir(history_dir)


def _list_history_files(history_dir):
    result = []
    for root, _, files in os.walk(history_dir):
        files.sort()
        result += [os.path.join(root, f) for f in files]

    return result


def _list_history_lines(history_dir):
    files = _list_history_files(history_dir)
    lines = []
    for file_ in files:
        with open(file_) as file_handler:
            lines.append(file_handler.read())

    return lines


def _check_history_dir(history_dir):
    try:
        os.stat(history_dir)
    except OSError:
        os.mkdir(history_dir)


def _delete_history_dir(history_dir):
    try:
        shutil.rmtree(history_dir)
    except OSError:
        pass


def _select_history_dir(args):
    filepath = os.path.join(os.getcwd(), args.history_file)
    return os.path.join(select_dir_with_file(filepath), args.history_dir)


def select_dir_with_file(filepath):
    dirname = os.path.dirname(filepath)
    filename = os.path.basename(filepath)
    try:
        return os.path.dirname(find_file_across_path(dirname, filename))
    except RuntimeError as error:
        raise RuntimeError('History file not found!', error)


def delete(args):
    history_dir = _select_history_dir(args)

    files = _list_history_files(history_dir)
    files = zip(_str_count(1), files)

    if args.entry:
        register = {item[0]: item[1] for item in files}
        for entry in args.entry:
            try:
                file_to_delete = register[entry]
                os.remove(file_to_delete)
            except KeyError:
                pass
    else:
        lines = []
        for entry in files:
            with open(entry[1]) as file_handler:
                lines.append('{}. {}'.format(entry[0], file_handler.read()))

        lines = chain(lines, ['\n', '(Delete by choosing entries numbers.)'])
        print('\n' + ''.join(lines))


def _str_count(start=None):
    items = count(start)
    for item in items:
        yield str(item)



#!/usr/bin/env python
# coding: utf-8

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from pyhistory import __version__, __author__, __email__


readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

requirements = []

setup(
    name='pyhistory',
    version=__version__,
    description='Package to help maintaining HISTORY file for Python project.',
    long_description=readme + '\n\n' + history,
    author=__author__,
    author_email=__email__,
    url='https://github.com/beregond/pyhistory',
    packages=[
        'pyhistory',
    ],
    package_dir={
        'pyhistory': 'pyhistory',
    },
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'pyhistory = pyhistory.cli:main',
            'pyhi = pyhistory.cli:main',
        ]
    },
    license="BSD",
    zip_safe=False,
    keywords='pyhistory',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
)
